# -*- coding: utf-8 -*-
# Copyright (C) 2008  Warp Networks, S.L.
# Author:  Jaime Soriano, Isaac Clerencia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import gtk

import dbus

from gtkmvc import Controller

import wader.common.consts as consts
from wader.gtk.logger import logger

from wader.gtk.dialogs import (show_profile_window, show_preferences_window,
                               show_warning_dialog, ActivityProgressBar,
                               show_about_dialog, 
                               ask_pin_dialog, ask_puk_dialog, ask_puk2_dialog)
from wader.gtk.utils import bytes_repr
from wader.gtk.consts import GTK_LOCK

class MainController(Controller):

    def __init__(self, model):
        super(MainController, self).__init__(model)
        model.ctrl = self
        self.cid = None

    def register_view(self, view):
        super(MainController, self).register_view(view)
        self._connect_to_signals()

    def _connect_to_signals(self):
        self.view['window1'].connect('delete_event', self.close_application)
        self.cid = self.view['connect_button'].connect('toggled',
                                            self.on_connect_button_toggled)

    def close_application(self, *args):
        if self.model.dial_path:
            show_warning_dialog("Cannot close application",
                                "Cannot close while a connection is active")
            window = self.view.get_top_widget()
            window.emit_stop_by_name('delete_event')
            return True
        else:
            self.model.quit(self._close_application_cb)

    def ask_for_new_profile(self):
        if self.model.device:
            self.model.get_imsi(lambda imsi:
                show_profile_window(self.model, imsi=imsi))
        else:
            show_profile_window(self.model)

    def _close_application_cb(self, *args):
        os.unlink(GTK_LOCK)
        gtk.main_quit()

    def property_rssi_value_change(self, model, old, new):
        self.view.rssi_changed(new)

    def property_operator_value_change(self, model, old, new):
        if new == 'Unknown':
            logger.error("Unknown operator received, using profile name...")
            opath = self.model.conf.get('profile', 'name')
            try:
                obj = self.model.bus.get_object(consts.WADER_PROFILES_SERVICE,
                                                opath)
                name = obj.Get('name')
                self.view.operator_changed(name)
            except dbus.DBusException, e:
                logger.error("Error while getting profile name %s" % e)
        else:
            self.view.operator_changed(new)

    def property_tech_value_change(self, model, old, new):
        self.view.tech_changed(new)

    def property_device_value_change(self, model, old, new):
        try:
            profile = model.conf.get('profile', 'name')
        except:
            profile = ""

    def property_profile_value_change(self, model, old, new):
        logger.info("A profile has been set for current model")

    def property_status_value_change(self, model, old, new):
        self.view.set_status(new)
        if new == 'No device':
            self.view.set_disconnected()

    def property_pin_required_value_change(self, model, old, new):
        if new:
            pin = ask_pin_dialog()
            if pin:
                model.send_pin(pin, model.enable_device)

    def property_puk_required_value_change(self, model, old, new):
        if new:
            puk = ask_puk_dialog()
            if puk:
                model.send_puk(puk, model.enable_device)

    def property_puk2_required_value_change(self, model, old, new):
        if new:
            puk2 = ask_puk2_dialog()
            if puk2:
                model.send_puk2(puk2, model.enable_device)

    def property_rx_bytes_value_change(self, model, old, new):
        if old != new:
            self.view['rx_bytes_label'].set_text(bytes_repr(new))
            logger.info("Bytes rx: %d", new)

    def property_tx_bytes_value_change(self, model, old, new):
        if old != new:
            self.view['tx_bytes_label'].set_text(bytes_repr(new))
            logger.info("Bytes tx: %d", new)

    def property_total_bytes_value_change(self, model, old, new):
        if old != new:
            self.view['total_bytes_label'].set_text(bytes_repr(new))
            logger.info("Total bytes: %d", new)
    
    def property_transfer_limit_exceeded_value_change(self, model, old, new):
        if not old and new:
            show_warning_dialog("Transfer limit exceeded", 
                                "You have exceeded your transfer limit")

    def _on_connect_cb(self, dev_path):
        self.view.set_connected()
        self.model.start_stats_tracking()
        if self.apb:
            self.apb.close()
            self.apb = None
        self.model.dial_path = dev_path

    def _on_connect_eb(self, args):
        self.view.set_disconnected()
        if self.apb:
            self.apb.close()
            self.apb = None
        logger.error(args)

    def _on_disconnect_cb(self):
        logger.info("Disconnected")
        self.model.stop_stats_tracking()
        self.view.set_disconnected()
        if self.apb:
            self.apb.close()
            self.apb = None

    def _on_disconnect_eb(self, args):
        self.model.stop_stats_tracking()        
        if self.apb:
            self.apb.close()
            self.apb = None
        logger.error(args)

    def on_connect_button_toggled(self, widget):
        bus = dbus.SessionBus()
        dialmanager = bus.get_object(consts.WADER_DIALUP_SERVICE,
                                     consts.WADER_DIALUP_OBJECT)

        if widget.get_active():
            # user wants to connect
            if not self.model.device:
                widget.set_active(False)
                show_warning_dialog("No device found", "No device has been found. Please, insert one and try again.")
                return

            if not self.model.profile:
                widget.set_active(False)
                show_warning_dialog("Profile needed", "A profile is needed for connecting, please, create it.")
                self.ask_for_new_profile()
                return

            self.apb = ActivityProgressBar("Connecting", self, disable_cancel=True)
            profile = self.model.profile.profile_path
            dialmanager.ActivateConnection(profile, self.model.device_path,
                                    dbus_interface=consts.DIL_INTFACE,
                                    reply_handler=self._on_connect_cb,
                                    error_handler=self._on_connect_eb)
            self.apb.init()
            logger.info("Connecting...")
        else:
            # user wants to disconnect
            if not self.model.dial_path:
                return

            self.apb = ActivityProgressBar("Disconnecting", self, disable_cancel=True)
            dialmanager.DeactivateConnection(self.model.device_path,
                                    dbus_interface=consts.DIL_INTFACE,
                                    reply_handler=self._on_disconnect_cb,
                                    error_handler=self._on_disconnect_eb)

            self.apb.init()
            self.model.dial_path = None

    def on_preferences_menu_item_activate(self, widget):
        show_preferences_window(self.model)

    def on_about_menuitem_activate(self, widget):
        dlg = show_about_dialog()
        dlg.run()
        dlg.destroy()

    def on_exit_menu_item_activate(self, widget):
        self.close_application()
