# -*- coding: utf-8 -*-
# Copyright (C) 2008  Warp Networks, S.L.
# Author:  Jaime Soriano, Isaac Clerencia, Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import gtk
import gobject
from gnome import url_show

import dbus

from twisted.internet import reactor
from wader.gtk.logger import logger
import wader.common.consts as consts
from wader.gtk.consts import APP_ARTISTS

def show_about_dialog():
    abt = gtk.AboutDialog()
    icon = abt.render_icon(gtk.STOCK_ABOUT, gtk.ICON_SIZE_MENU)
    abt.set_icon(icon)

    gtk.about_dialog_set_url_hook(lambda abt, url: url_show(url))
    gtk.about_dialog_set_email_hook(lambda d, e: url_show("mailto:%s" % e))

    abt.set_program_name(consts.APP_NAME)
    abt.set_version(consts.APP_VERSION)
    abt.set_copyright("""
Warp Networks S.L.
Elonex
Paul Hardwick
""")
    abt.set_authors(consts.APP_AUTHORS)
    abt.set_documenters(consts.APP_DOCUMENTERS)
    abt.set_artists(APP_ARTISTS)
    abt.set_website(consts.APP_URL)

    abt.set_website_label('http://public.warp.es/wader')
    _license = """
The Wader project
Copyright (C) 2008  Warp Networks, S.L.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA"""
    abt.set_license(_license)
    return abt


def show_profile_window(main_model, profile_path=None, profile=None, imsi=None):
    from wader.gtk.models.profile import ProfileModel
    from wader.gtk.controllers.profile import ProfileController
    from wader.gtk.views.profile import ProfileView

    if profile:
        model = profile
    else:
        model = ProfileModel(main_model, profile_path, imsi)
    controller = ProfileController(model)
    view = ProfileView(controller)
    view.show()

def show_preferences_window(main_model):
    from wader.gtk.models.preferences import PreferencesModel
    from wader.gtk.views.preferences import PreferencesView
    from wader.gtk.controllers.preferences import PreferencesController

    model = PreferencesModel(main_model)
    controller = PreferencesController(model)
    view = PreferencesView(controller)
    view.show()

def make_basic_dialog(title, buttons, stock_image):
    flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT | \
            gtk.DIALOG_NO_SEPARATOR
    dialog = gtk.Dialog(title, flags=flags, buttons=buttons)
    box = gtk.HBox()
    box.add(gtk.image_new_from_stock(stock_image, gtk.ICON_SIZE_DIALOG))
    vbox = gtk.VBox()
    box.add(vbox)

    box.set_spacing(12)
    vbox.set_spacing(6)

    alignment = gtk.Alignment()
    alignment.set_padding(6,6,6,6)
    alignment.add(box)
    dialog.vbox.add(alignment)
    return dialog, vbox

def show_warning_dialog(title, message):
    logger.warn("Warning dialog: %s" % message)

    buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK)
    dialog, box = make_basic_dialog(title, buttons, gtk.STOCK_DIALOG_WARNING)
    box.add(gtk.Label(message))
    dialog.set_default_response(gtk.RESPONSE_OK)

    dialog.show_all()
    ret = dialog.run()
    dialog.destroy()
    return ret

def show_error_dialog(title, message):
    logger.error("Error dialog: %s" % message)

    buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK)
    dialog, box = make_basic_dialog(title, buttons, gtk.STOCK_DIALOG_ERROR)
    box.add(gtk.Label(message))
    dialog.set_default_response(gtk.RESPONSE_OK)

    dialog.show_all()
    ret = dialog.run()
    dialog.destroy()
    return ret

def generic_auth_dialog(title, message):
    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
               gtk.STOCK_OK, gtk.RESPONSE_OK)

    dialog, box = make_basic_dialog(title, buttons, gtk.STOCK_DIALOG_QUESTION)
    box.add(gtk.Label(message))
    entry = gtk.Entry()
    box.add(entry)
    dialog.set_default_response(gtk.RESPONSE_OK)

    dialog.show_all()
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        ret = entry.get_text()
    else:
        ret = None
    dialog.destroy()
    return ret

def ask_pin_dialog():
    logger.info("Asking for PIN")    
    return generic_auth_dialog("PIN Required", "Please, insert the PIN of your SIM card")

def ask_puk_dialog():
    logger.info("Asking for PUK")        
    return generic_auth_dialog("PUK Required", "Please, insert the PUK of your SIM card")

def ask_puk2_dialog():
    logger.info("Asking for PUK2")
    return generic_auth_dialog("PUK2 Required", "Please, insert the PUK2 of your SIM card")

PULSE_STEP = .2
MAX_WIDTH = 260
MIN_WIDTH = 150
SIZE_PER_CHAR = 15

class ActivityProgressBar(object):
    """
    I am an activity progress bar

    useful for situation where we don't beforehand how long wil
    take an IO operation to complete
    """
    def __init__(self, title, parent, initnow=False, disable_cancel=False):
        self.window = None
        self.parent = parent
        self.vbox = None
        self.bbox = None
        self.progress_bar = None
        self.cancel_button = None
        self.loop = None
        self.default_func = None
        self.default_args = None
        self.default_call = None
        self.cancel_func = None
        self.cancel_args = None

        self._build_gui(title, parent, disable_cancel)
        if initnow:
            self.init()

    def _build_gui(self, title, parent, disable_cancel):
        self.window = gtk.Window()
        self.window.set_transient_for(parent.view.get_top_widget())
        self.window.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.window.set_resizable(False)
        self.window.set_deletable(False)
        self.window.set_modal(True)

        width = max(MIN_WIDTH, min(len(title) * SIZE_PER_CHAR, MAX_WIDTH))
        self.window.set_size_request(width, -1)
        self.window.set_title(title)
        self.window.set_border_width(0)
        self.window.connect('delete_event', self.on_delete_event)

        self.vbox = gtk.VBox()

        self.progress_bar = gtk.ProgressBar()
        self.progress_bar.set_pulse_step(PULSE_STEP)

        self.vbox.pack_start(self.progress_bar)

        self.bbox = gtk.HButtonBox()
        self.bbox.set_layout(gtk.BUTTONBOX_END)
        self.cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
        if disable_cancel:
            self.cancel_button.set_sensitive(False)
        else:
            self.bbox.add(self.cancel_button)

        self.vbox.pack_end(self.bbox, False, False)
        self.window.add(self.vbox)

        self.exit = None

        self.cancel_button.connect('clicked', self.on_cancel_button_clicked)

    def init(self):
        self.progress_bar.show_all()
        gobject.timeout_add(200, self.pulse_update)
        self.bbox.show_all()
        self.window.show_all()
        self.connect_to_signals()

    def connect_to_signals(self):
        bus = dbus.SessionBus()
        self.disconnect_sm = bus.add_signal_receiver(
                                self.disconnected_cb,
                                "Disconnected",
                                consts.DIL_INTFACE)

    def disconnected_cb(self):
        """
        Executed when a e.w.W.Dial.Disconnect signal is received
        """
        logger.error("Disconnected received")
        # untoggle connect button
        ctrl = self.parent
        button = ctrl.view['connect_button']
        # block handler
        button.handler_block(ctrl.cid)
        button.set_active(False)
        # unblock it
        button.handler_unblock(ctrl.cid)
        self.close()

    def pulse_update(self):
        if self.exit:
            return False
        else:
            self.progress_bar.pulse()
            return True

    def close(self):
        self.window.destroy()
        self.exit = True
        if self.disconnect_sm:
            # do not listen for es.warp.Wader.Dialer.Disconnect signals
            self.disconnect_sm.remove()

    def on_delete_event(self, *args):
        return True

    def on_cancel_button_clicked(self, widget):
        if self.default_call:
            if self.default_call.active:
                self.default_call.cancel()

        if self.cancel_func:
            self.cancel_func(*self.cancel_args)

        self.close()

    def set_default_cb(self, delay, func, *args):
        self.default_func = func
        self.default_args = args
        def execute_and_close():
            self.default_func(*self.default_args)
            self.close()

        self.default_call = reactor.callLater(delay, execute_and_close)

    def set_cancel_cb(self, func, *args):
        self.cancel_func = func
        self.cancel_args = args
