import dbus
import gobject

from gtkmvc import Model

from wader.gtk.logger import logger
from wader.gtk.utils import dbus_error_is

from wader.common.configbase import WaderConfigBase
from wader.common.consts import (WADER_SERVICE, WADER_OBJPATH, WADER_INTFACE,
                                 WADER_DIALUP_SERVICE, WADER_DIALUP_OBJECT,
                                 CRD_INTFACE, NET_INTFACE, PIN_INTFACE,
                                 WADER_CFG)

from wader.gtk.models.profile import ProfileModel

import wader.common.exceptions as ex

class MainModel(Model):

    __properties__ = {
        'rssi' : 0,
        'profile' : None,
        'device' : None,
        'device_path' : None,
        'dial_path' : None,
        'operator' : 'Unknown',
        'status' : 'Not registered',
        'tech' : '----',

        'pin_required': False,
        'puk_required': False,
        'puk2_required': False,

        'rx_bytes': 0,
        'tx_bytes': 0,
        'total_bytes': 0,

        'transfer_limit_exceeded': False
    }

    def __init__(self):
        super(MainModel, self).__init__()
        self.bus = dbus.SessionBus()
        self.obj = None
        self.conf = None
        self.device_path = ""
        self.dial_manager = None
        self.tracking_stats = False
        self._get_wader_obj()
        self._get_device()
        self._get_config()
        self.obj.connect_to_signal("DeviceAdded", self._device_added_cb)
        self.obj.connect_to_signal("DeviceRemoved", self._device_removed_cb)

    def _device_added_cb(self, udi):
        logger.info('Device with udi %s added' % udi)
        self.obj.GetDevices(dbus_interface=WADER_INTFACE,
                            reply_handler=self._get_devices_cb,
                            error_handler=self._get_devices_eb)

    def _device_removed_cb(self, udi):
        logger.info('Device with udi %s removed' % udi)
        if self.device_path:
            logger.info('Device path: %s' % self.device_path)
        if udi == self.device_path:
            self.device_path = None
            self.dial_path = None
            self.operator = 'Unknown'
            self.status = 'No device'
            self.tech = '----'
            self.rssi = 0

    def quit(self, quit_cb):
        if self.device_path and self.obj:
            logger.info("Removing device %s before quit." % self.device_path)
            self.obj.RemoveDevice(
                self.device_path,
                dbus_interface=WADER_INTFACE,
                reply_handler=quit_cb,
                error_handler=lambda error: \
                   logger.error("Cannot Remove Device:\n" + error.message)) or \
                   quit_cb()
        else:
            quit_cb()

    def get_imsi(self, cb):
        self.device.GetIMSI(
            dbus_interface=CRD_INTFACE,
            reply_handler=lambda imsi: cb(imsi),
            error_handler=lambda error:\
               logger.error(
                    "Cannot get imsi for device %s" % self.device_path) or\
               cb(None))

    # Get Wader Object
    def _get_wader_obj(self):
        self.obj = self.bus.get_object(WADER_SERVICE, WADER_OBJPATH)

    # Get Device
    def _get_device(self):
        self.obj.GetDevices(dbus_interface=WADER_INTFACE,
                            reply_handler=self._get_devices_cb,
                            error_handler=self._get_devices_eb)

    def _get_devices_eb(self, error):
        logger.error(error)

    def _get_devices_cb(self, opaths):
        if not len(opaths):
            logger.warn("No devices found")
            return

        self.device_path = opaths[0]
        logger.info("Adding device %s" % self.device_path)

        self.obj.AddDevice(self.device_path,
                           dbus_interface=WADER_INTFACE,
                           reply_handler=self._add_device_cb,
                           error_handler=self._add_device_eb)

    def _add_device_cb(self, opath):
        logger.info("Device added %s" % opath)
        self.device = self.bus.get_object(WADER_SERVICE, opath)
        self.pin_required = False
        self.puk_required = False
        self.puk2_required = False
        self.total_bytes = self.conf.getint('statistics', 'total_bytes')
        self.enable_device()

    def enable_device(self):
        self.device.Enable(dbus_interface=CRD_INTFACE,
                         reply_handler=self._enable_device_cb,
                         error_handler=self._enable_device_eb)

    def _add_device_eb(self, error):
        if dbus_error_is(error, ex.DeviceAlreadyAddedError):
            logger.warn("Device already added, trying to enable...")
            self._add_device_cb(self.device_path)
        else:
            logger.error("Cannot add device:\n%s" % error.message)

    def _enable_device_cb(self):
        logger.info("Device enabled")

        self.pin_required = False
        self.puk_required = False
        self.puk2_required = False

        self.device.connect_to_signal("RSSIChanged", self._rssi_changed_cb)
        self.device.connect_to_signal("CellTypeChanged",
                                      self._cell_type_changed_cb)

        self.device.GetSignal(dbus_interface=NET_INTFACE,
                              reply_handler=self._rssi_changed_cb,
                              error_handler=lambda m: \
                                logger.warn("Cannot get RSSI %s" % m))

        self.device.GetInfo(dbus_interface=NET_INTFACE,
                            reply_handler=self._get_netinfo_cb,
                            error_handler=lambda m: \
                                logger.warn("Cannot get network information"))

        self._get_regstatus(first_time=True)

    def _get_regstatus(self, first_time=False):
        self.device.GetRegistrationStatus(dbus_interface=NET_INTFACE,
                            reply_handler=self._get_regstatus_cb,
                            error_handler=lambda error: \
                                logger.warn("Cannot get registration status"+\
                                            error.message))

        if not first_time and self.status != "Scanning":
            return False

    def _enable_device_eb(self, error):
        logger.warn("Cannot enable device:\n%s" % error.message)
        if dbus_error_is(error, ex.CMEErrorSIMPINRequired):
            self.pin_required = True
        elif dbus_error_is(error, ex.CMEErrorSIMPUKRequired):
            self.puk_required = True
        elif dbus_error_is(error, ex.CMEErrorSIMPUK2Required):
            self.puk2_required = True

    # Get Configuration
    def _get_config(self):
        if not self.conf:
            self.conf = WaderConfigBase(WADER_CFG)
        
        profile_path = self.conf.get('profile', 'name')
        if profile_path and not self.profile:
            try:
                self.profile = ProfileModel(self, profile_path)
            except:
                logger.warn("Cannot load initial profile, trying again...")
                gobject.timeout_add(1000, self._get_config)

    def _rssi_changed_cb(self, rssi):
        logger.info("RSSI changed %d" % rssi)
        self.rssi = rssi

    def _get_regstatus_cb(self, (active, status)):
        if status == 1:
            self.status = "Registered"
        elif status == 2:
            self.status = "Scanning"
            # get the registration status again in 5s
            gobject.timeout_add(5000, self._get_regstatus)
        elif status == 3:
            self.status = "Reg. rejected"
        elif status == 4:
            self.status = "Unknown Error"
        elif status == 5:
            self.status = "Roaming"

    def _get_netinfo_cb(self, info):
        self.operator = info[0].split(' ')[0]
        self.tech = info[1] == '3G' and "UMTS" or "GPRS"
        logger.info("Operator: %s" % self.operator)
        logger.info("Technology: %s" % self.tech)

    def _cell_type_changed_cb(self, cell_type):
        logger.info("Cell type changed %s" % cell_type)
        self.tech = cell_type.split('_')[0]

    def send_pin(self, pin, cb):
        logger.info("Trying authentication with PIN %s" % pin)
        self.device.Send(pin, 
                         dbus_interface=PIN_INTFACE,
                         reply_handler=lambda *args: cb(),
                         error_handler=lambda m: \
                                logger.error("Cannot authenticate with PIN"))

    def send_puk(self, puk, cb):
        logger.info("Trying authentication with PUK %s" % puk)
        self.device.SendPUK(puk, 
                         dbus_interface=PIN_INTFACE,
                         reply_handler=lambda *args: cb(),
                         error_handler=lambda m: \
                                logger.error("Cannot authenticate with PIN"))


    def send_puk2(self, puk2, cb):
        logger.error("Trying authentication with PUK2, not implemented yet")

    def profile_added(self, profile):
        if not self.profile:
            self.profile = profile
            self.profile.main_model = self
            self.conf.set('profile', 'name', self.profile.profile_path)
            self.conf.write()

    def profile_updated(self, profile):
        if self.profile and self.profile.profile_path == profile.profile_path:
            self.profile = profile
            self.profile.main_model = self
        if not self.profile:
            self.profile = profile
            self.profile.main_model = self
            self.conf.set('profile', 'name', self.profile.profile_path)
            self.conf.write()

    def profile_removed(self, profile):
        if self.profile and self.profile.profile_path == profile.profile_path:
            self.profile = None
            self.conf.set('profile', 'name', '')
            self.conf.write()

    def profile_default(self, profile):
        self.profile = ProfileModel(self, profile_path=profile.profile_path)
        self.conf.set('profile', 'name', profile.profile_path)
        self.conf.write()

    def check_transfer_limit(self):
        warn_limit = self.conf.get('statistics', 'warn_limit') == "yes"
        if warn_limit:
            transfer_limit = self.conf.get('statistics', 'transfer_limit')
            transfer_limit = float(transfer_limit) * 2**20
            if self.total_bytes > transfer_limit:
                self.transfer_limit_exceeded = True
            else:
                self.transfer_limit_exceeded = False
        else:
            self.transfer_limit_exceeded = False

    def update_stats(self):
        if self.dial_path and self.dial_manager:
            stats = self.dial_manager.GetStats(self.device_path)
            self.rx_bytes, self.tx_bytes = stats
            self.total_bytes = \
                self.conf.getint('statistics', 'total_bytes') + \
                self.rx_bytes + self.tx_bytes
            self.check_transfer_limit()
        if self.tracking_stats:
            gobject.timeout_add(2000, self.update_stats)

    def start_stats_tracking(self):
        if not self.dial_manager:
            self.dial_manager = self.bus.get_object(WADER_DIALUP_SERVICE,
                                                    WADER_DIALUP_OBJECT)
        self.tracking_stats = True
        self.update_stats()

    def stop_stats_tracking(self):
        self.rx_bytes = 0
        self.tx_bytes = 0
        self.conf.set('statistics', 'total_bytes', str(self.total_bytes))
        self.conf.write()
        self.tracking_stats = False
