# -*- coding: utf-8 -*-
# Copyright (C) 2008  Warp Networks, S.L.
# Author:  Jaime Soriano
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import dbus

from gtkmvc import Model

from wader.common.consts import (WADER_PROFILES_SERVICE, WADER_PROFILES_OBJECT,
                                 PRF_INTFACE)
from wader.gtk.models.profile import ProfileModel
from wader.gtk.logger import logger

PREF_TABS = ["PROFILES"]

class PreferencesModel(Model):

    __properties__ = {
        'current_tab': PREF_TABS[0],

        'default_profile': None,
        'profiles': None,

        'warn_limit' : False,
        'transfer_limit' : -1
    }

    def __init__(self, main_model):
        super(PreferencesModel, self).__init__()
        self.bus = None
        self.main_model = main_model
        self.profiles_manager = None

        try:
            self.default_profile = self.main_model.profile
        except:
            self.default_profile = None

        self._get_profile_manager()

    def _get_profile_manager(self):
        self.bus = dbus.SessionBus()
        self.manager = self.bus.get_object(WADER_PROFILES_SERVICE,
                                           WADER_PROFILES_OBJECT)

    def get_profiles(self):
        if not self.profiles:
            profiles = self.manager.GetProfiles(dbus_interface=PRF_INTFACE)
            ret = {}
            for profile_path in profiles:
                ret[profile_path] = ProfileModel(self, profile_path)
            self.profiles = ret

        return self.profiles

    def get_profile_by_path(self, path):
        return self.get_profiles(self)[path]

    def load(self):
        conf = self.main_model.conf
        def try_to_get(section, key, default):
            ret = default
            try:
                ret = conf.get(section, key)
            except Exception, e:
                logger.err(e.message)
            return ret

        self.warn_limit = \
                try_to_get('statistics', 'warn_limit', 'yes') == 'yes'
        self.transfer_limit = \
                float(try_to_get('statistics', 'transfer_limit', 50.0))

    def save(self):
        conf = self.main_model.conf
        conf.set('statistics', 'warn_limit', self.warn_limit and "yes" or "no")
        conf.set('statistics', 'transfer_limit', str(self.transfer_limit))
        conf.write()

    def reset_statistics(self):
        logger.info('Resetting total bytes')
        self.main_model.total_bytes = 0
        self.main_model.conf.set('statistics', 'total_bytes', '0')
        self.main_model.conf.write()

    def profile_added(self, profile):
        new_profiles = dict(self.profiles)
        new_profiles[profile.profile_path] = profile
        self.profiles = new_profiles
        self.main_model.profile_added(profile)

    def profile_updated(self, profile):
        new_profiles = dict(self.profiles)
        new_profiles[profile.profile_path] = profile
        self.profiles = new_profiles
        self.main_model.profile_updated(profile)

    def profile_removed(self, profile):
        if self.default_profile and \
           self.default_profile.profile_path is profile.profile_path:
            self.default_profile = None
        if profile.profile_path and \
           self.profiles.has_key(profile.profile_path):
            new_profiles = dict(self.profiles)
            del(new_profiles[profile.profile_path])
            self.profiles = new_profiles
            self.main_model.profile_removed(profile)

    def profile_default(self, profile):
        self.default_profile = profile
        self.main_model.profile_default(profile)
