# -*- coding: utf-8 -*-
# Copyright (C) 2008  Warp Networks, S.L.
# Author:  Jaime Soriano
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import dbus

from gtkmvc import Model

from wader.common.consts import (WADER_PROFILES_SERVICE, WADER_PROFILES_OBJECT,
                                 PRF_INTFACE)
from wader.common.hardware import CONN_OPTS, CONN_OPTS_REV

from wader.gtk.logger import logger
from wader.gtk.validations import *

class ProfileModel(Model):

    __properties__ = {
        'name' : "",
        'username' : "",
        'password' : "",
        'connection' : "",
        'apn' : "",
        'static_dns': False,
        'primary_dns' : "",
        'secondary_dns' : ""
    }

    def __init__(self, parent_model, profile_path=None, imsi=None):
        super(ProfileModel, self).__init__()
        self.bus = None
        self.manager = None
        self.profile_path = None
        self.parent_model = parent_model
        self._get_profile_manager()
        #self._init_validations()

        if profile_path:
            self._get_profile(profile_path)

        if imsi:
            self._get_provided_profile(imsi)

    def _init_validations(self):
        self.validator = ModelValidator(self)
        self.validator.add_validation(
            'name', validate_non_empty_string,
            "Please, insert a profile name")
        self.validator.add_validation(
            'username', validate_non_empty_string,
            "Please, insert a non empty username")
        self.validator.add_validation(
            'password', validate_non_empty_string,
            "Please, instert a non empty password")
        self.validator.add_validation(
            'apn', validate_non_empty_string,
            "Please, insert a valid APN")

        def dns_validator(value):
            if not self.static_dns:
                return True
            else:
                return validate_ip(value)

        self.validator.add_validation(
            'primary_dns', dns_validator,
            "Please, insert a valid primary DNS")
        self.validator.add_validation(
            'secondary_dns', dns_validator,
            "Please, insert a valid secondary DNS")

    def _get_profile_manager(self):
        # Initializes dbus objects
        self.bus = dbus.SessionBus()
        self.manager = self.bus.get_object(WADER_PROFILES_SERVICE,
                                           WADER_PROFILES_OBJECT)

    def _get_profile(self, profile_path):
        # Tries to load a known profile
        try:
            logger.info("Loading profile %s" % profile_path)
            profile = self.bus.get_object(WADER_PROFILES_SERVICE,
                                          profile_path)
            self.profile_path = profile_path

            def try_to_get(key, default):
                ret = default
                try:
                    ret = profile.Get(key)
                except:
                    pass
                return ret

            self.name = try_to_get('name', 'Unknown profile')
            self.username = try_to_get('username', '')
            self.password = try_to_get('password', '')
            self.connection = try_to_get('connection', CONN_OPTS[0])
            self.apn = try_to_get('apn', '')
            self.primary_dns = try_to_get('dns1', '')
            self.secondary_dns = try_to_get('dns2', '')
            default_static_dns = (self.primary_dns and self.secondary_dns) and '1' or '0'
            self.static_dns = (try_to_get('staticdns', default_static_dns) == "1")

        except Exception, e:
            logger.warn("Cannot load profile %s, creating a new one..." % \
                        profile_path)
            logger.warn(e.message)

    def _get_provided_profile(self, imsi):
        try:
            logger.info("Loading profile for imsi %s" % str(imsi))
            profile = self.manager.CreateProfileFromIMSI(imsi[:5],
                                                    dbus_interface=PRF_INTFACE)
            self._get_profile(profile)
            self.delete()
        except Exception, e:
            logger.warn("Cannot create profile for imsi %s, creating a new one..." % \
                        str(imsi))
            logger.warn(e.message)


    def save(self):
        opts = {
            'name'     : self.name,
            'username' : self.username,
            'password' : self.password,
            'apn' : self.apn,
            'connection' : self.connection,
            'staticdns' : self.static_dns and "1" or "0",
            'dns1' : self.primary_dns,
            'dns2' : self.secondary_dns
        }
        #self.validator.check()
        if self.profile_path:
            self.manager.ModifyProfile(self.profile_path,
                                       opts,
                                       dbus_interface=PRF_INTFACE)
            logger.info("Profile modified: %s" % self.profile_path)
            self.parent_model.profile_updated(self)
        else:
            self.profile_path = \
                self.manager.CreateProfile(opts, dbus_interface=PRF_INTFACE)
            logger.info("Profile added: %s" % self.profile_path)
            self.parent_model.profile_added(self)

        from wader.gtk.models.preferences import PreferencesModel
        if isinstance(self.parent_model, PreferencesModel):
            obj = self.parent_model.main_model.obj
        else:
            obj = self.parent_model.obj

        opts = dict(conn=CONN_OPTS_REV[self.connection])
        # says ConfigureConnection method is not found, wtf!
        #obj.ConfigureConnection(opts,
        #                        dbus_interface=CRD_INTFACE)

    def delete(self):
        if self.profile_path:
            logger.info("Removing profile %s" % self.profile_path)
            self.manager.DeleteProfile(self.profile_path,
                                       dbus_interface=PRF_INTFACE)
            logger.info("Profile removed: %s" % self.profile_path)
            self.parent_model.profile_removed(self)
            self.profile_path = None
        else:
            logger.warn("Trying to remove an unsaved profile")

    def set_as_default(self):
        if self.profile_path:
            logger.info("Setting profile %s as default" % self.profile_path)
            self.parent_model.profile_default(self)
        else:
            logger.warn("Trying to set an unsaved profile as default")
